import numpy as np
import projlibmc
import time
import matplotlib.pyplot as plt
import scipy
import atomwrite 
UseVisual = False
import atomvis
seedval = int(time.clock())
np.random.seed(seedval)

def InitPositions(NN, L):
    N = 2*(2*NN+1)*(2*NN+1)    
    Pos = np.zeros((N,3), float)
    Posid = np.zeros((N,1), int)
    Posvec = np.array([[0.0, 0.0, 0.0],[2/3., 1/3., -1./np.sqrt(6)]])
    a = 5.64 / np.sqrt(2)
    Rotmat = a*np.array([[1., -0.5, 0.], [0., np.sqrt(0.75), 0.], [0., 0., 1.]])
    for i in range(-NN, NN+1):
        for j in range(-NN, NN+1):
            indx = 2*(2*NN+1)*(i+NN) + 2*(j+NN)
            Pos[indx,:] = Posvec[0,:] + [float(i), float(j), 0.]            
            Pos[indx,:] = np.dot(Pos[indx,:], Rotmat.transpose())
            Pos[indx,:] += [0.0, 0.0, -0.85*L]
            Posid[indx] = 1
            Pos[indx+1,:] = Posvec[1,:] + [float(i), float(j), 0.]            
            Pos[indx+1,:] = np.dot(Pos[indx+1,:], Rotmat.transpose())
            Pos[indx+1,:] += [0.0, 0.0, -0.85*L]
            Posid[indx+1] = 2
    NeighborList = CalcNeighborList(Pos, a, N, L)
    return Pos, Posid, NeighborList

def CalcNeighborList(Pos, a, N, L):
    NList = []
    for i in range(0, N, 2):
        Posi = Pos[i,:]
        for j in range(1, N+1, 2):
            rij = Pos[j,:] - Posi
            rij[0] = rij[0] - L*round(rij[0]/L)
            rij[1] = rij[1] - L*round(rij[1]/L)
            rsq = np.sum(rij*rij)
            errval = abs(1.0 - (2.*rsq / (a*a)))                
            if errval < 0.01:
                NList.append([i, j])
    NList = np.array(NList, dtype = int)
    return NList

def CalcNeighborList1(Pos, a, NNa, NCl, L):
    NList = []
    for i in range(NNa):
        Posi = Pos[i,:]
        for j in range(NNa, NNa+NCl):
            rij = Pos[j,:] - Posi
            rsq = np.sum(rij*rij)
            errval = abs(1.0 - (2.*rsq / (a*a)))                
            if errval < 0.05:
                NList.append([i, j])
    NList = np.array(NList, dtype = int)
    return NList

def CalcRDF(NaList, ClList, NaClList, Nrbin, rbinvec, Nacount, Clcount, NaClcount):
    for i in range(Nrbin-1):
        for val1 in NaList:
            if val1 >= rbinvec[i] and val1 < rbinvec[i+1]:
                Nacount[i] += 1
        for val2 in ClList:
            if val2 >= rbinvec[i] and val2 < rbinvec[i+1]:
                Clcount[i] += 1
        for val3 in NaClList:
            if val3 >= rbinvec[i] and val3 < rbinvec[i+1]:
                NaClcount[i] += 1
    return Nacount, Clcount, NaClcount

def NucCoord(Nedge, L, Zval):
    N_Na = Nedge*(Nedge-1) / 2
    N_Cl = Nedge*(Nedge+1) / 2
    Posvec = np.array([[0.0, 0.0, 0.0],[1/3., 2/3., -1./np.sqrt(6)]])
    direc = np.array([[0., -1., 0.], [-1., 0., 0.], [1., 1., 0.]])    
    PosNa = np.zeros([N_Na,3], dtype = float)
    PosCl = np.zeros([N_Cl,3], dtype = float)
    Nacount = 0
    Clcount = 0    
    PosNa[Nacount,:] = Posvec[0,:]
    PosCl[Clcount,:] = Posvec[1,:]
    for i in range(3):
        Posprev1 = PosNa[Nacount,:]
        Posprev2 = PosCl[Clcount,:]
        direci = direc[i,:]
        for j in range(Nedge-1):
            if i<2 or j<Nedge-2:
                Clcount += 1
                PosCl[Clcount,:] = Posprev2 + direci
                Posprev2 = PosCl[Clcount,:]
        for j in range(Nedge-2):
            if i<2 or j<Nedge-3:
                Nacount += 1
                PosNa[Nacount,:] = Posprev1 + direci
                Posprev1 = PosNa[Nacount,:]
    NucPos = np.zeros([N_Na+N_Cl,3], dtype = float)
    Nucid = np.zeros([N_Na+N_Cl,1], dtype = int)
    a = 5.64 / np.sqrt(2)
    Rotmat = a*np.array([[1., -0.5, 0.], [0., np.sqrt(0.75), 0.], [0., 0., 1.]])
    for indx, val in enumerate(PosNa):
        NucPos[indx,:] = np.dot(val, Rotmat.transpose()) + [0.0, 0.0, Zval-0.85*L]
        Nucid[indx] = 1
    for indx, val in enumerate(PosCl):
        NucPos[Nacount+1+indx,:] = np.dot(val, Rotmat.transpose()) + [0.0, 0.0, Zval-0.85*L]
        Nucid[Nacount+1+indx] = 2
    NucList = CalcNeighborList1(NucPos, a, Nacount+1, Clcount+1, L)    
    return NucPos, Nucid, NucList    

#### YOUR CODE HERE ####

NN = 6
Cut = 25
kB = 8.617e-5       # eV/K
convfact1 = 1./9648.53336   # units of eV per (amu * Angstrom ^2 / picosecond^2)
mass = 0.5 * 58.44 * convfact1      #atomic mass units
N = 2*(2*NN+1)**2           # total number of atoms in the bilayer
Dim = 3
L = (5.64/np.sqrt(2))*(2*NN+1)        # Box length in Angstrom
T = 300.0           # T in K
EFracTolLS = 1.e-5
EFracTolCG = 1.e-6
dx = 0.01
drmax = 0.025
Nmill = 1000000
Ntotstep = 10*Nmill
Neqbm = 5*Nmill
sigm = np.zeros([2,2], dtype = float)
sigm[0,0] = 2.35
sigm[1,1] = 4.40
sigm[0,1] = 0.5*(sigm[0,0] + sigm[1,1])
sigm[1,0] = sigm[0,1]
epsi = np.zeros([2,2], dtype = float)
epsi[0,0] = 4. * 65.42*kB / 14.3990
epsi[1,1] = 4. * 50.32*kB / 14.3990
epsi[0,1] = np.sqrt(epsi[0,0]*epsi[1,1])
epsi[1,0] = epsi[0,1]
zdelta = 0.04        # angstrom
Nbin = 200
zbinvec = np.zeros(Nbin, dtype = float)
for i in range(Nbin):
    zbinvec[i] = zdelta * float(i)
Lvec = [1.*L, 1.*L, 2.*L]
#Volume = L * L * 2.0
#Nobs = 0
#rdelta = 0.05        #bin width in angstroms
#Nrbin = 600
#rbinvec = np.zeros(Nrbin, dtype = float)
#for i in range(Nrbin):
#    rbinvec[i] = rdelta * float(i)
#Nacount = np.zeros(Nrbin-1, dtype = float)
#Clcount = np.zeros(Nrbin-1, dtype = float)
#NaClcount = np.zeros(Nrbin-1, dtype = float)
#    Pos0 = Pos.copy()
Nedge = 3       #no. of Cl atoms in the triangular nucleus edge
NNa = Nedge*(Nedge-1) / 2
NCl = Nedge*(Nedge+1) / 2
Umax = 1.0e6            #eV
Zval = 6.0
Zvvec = np.array([1.5])
niter = len(Zvvec)
countmat = np.zeros([Nbin, niter], dtype = float)
for indx, Zj in enumerate(Zvvec):
    Pos, Posid, NList = InitPositions(NN, L)
    Zsurf = Pos[0,2]
    PEtot = 0.0    
    dzmax = 0.15*Zval/8.    
    PEvec = np.zeros(Ntotstep/1000, dtype = float)
    Tvec = np.zeros(Ntotstep/1000, dtype = float)
    PEcount = 0
    zcount = np.zeros(Nbin, dtype = float)
    NucPos, Nucid, NucList = NucCoord(Nedge, L, Zval)
    N1 = N * (N-2) / 8
    N2 = N * N / 4
    Ntot = N + NNa + NCl
    NucList = NucList + [N, N]
    NList = np.concatenate((NList, NucList))
    Pos = np.concatenate((Pos, NucPos))
    Posid = np.concatenate((Posid, Nucid))
    Names = []
    for elem in Posid:
        if elem == 1:
            Names.append('Na')
        else:
            Names.append('Cl')
    Pdb = atomwrite.pdbfile("anim.pdb",Lvec, Names, Compressed = False)
    PEtot = projlibmc.calcenergy(Pos, Posid, NList, Cut, L, N)
    count = 0
    Naccept = 0
    Nreject = 0
    Zavg = np.mean(Pos[N:Ntot,2])-Zsurf
    Beta = -1.0 / (kB * T)
    Nnuc = NNa + NCl
    Kconst = 0.9 / Nnuc
    Pdb.write(Pos)
    Atomcolor = []
    Atomradius = []
    for i in range(N):
        if i%2==0:
            Atomcolor.append((0.55,0.55,0.55))
            Atomradius.append(1.0)
        else:
            Atomcolor.append((0,1,0))
            Atomradius.append(1.5)
    for i in range(N, N+NNa):
        Atomcolor.append((0.55,0.55,0.55))
        Atomradius.append(1.0)
    for i in range(N+NNa, N+NNa+NCl):
        Atomcolor.append((0,1,0))
        Atomradius.append(1.5)
    atomvis.Init(Pos, Lvec, Atomcolor, Atomradius) 
    while count < Ntotstep:
        if count % 2 ==0:
            MoveAtom = np.random.randint(N, Ntot, size = 1)
            if Zavg < 2.0:
                Displacement = np.array([0.1*drmax*(2*np.random.random_sample() - 1.), 0.1*drmax*(2*np.random.random_sample() - 1.), 0.1*dzmax*(2*np.random.random_sample() - 1.)])        
            else:
                Displacement = np.array([0.1*drmax*(2*np.random.random_sample() - 1.), 0.1*drmax*(2*np.random.random_sample() - 1.), dzmax*(2*np.random.random_sample() - 1.)])        
            dEta = Kconst*(2.*(np.mean(Pos[N:Ntot,2]) - Zsurf - Zj) + Displacement[2]/Nnuc)*Displacement[2]
        else:
            MoveAtom = np.random.randint(0, N, size = 1)
            Displacement = np.array([drmax*(2*np.random.random_sample() - 1.), drmax*(2*np.random.random_sample() - 1.), 0.0])
            dEta = 0.0
        OldPos = Pos[MoveAtom,:].copy()
        U1 = projlibmc.calcenergymc(Pos, Posid, NucList, Cut, L, MoveAtom, N, sigm, epsi)
        Pos[MoveAtom,:] += Displacement 
        U2 = projlibmc.calcenergymc(Pos, Posid, NucList, Cut, L, MoveAtom, N, sigm, epsi)
        delU = U2 - U1 + dEta
        if abs(delU) < Umax and np.exp(Beta*delU) > np.random.random_sample():
            Naccept += 1
            PEtot += delU
        else:
            Nreject += 1
            Pos[MoveAtom,:] = OldPos
        Zavg = np.mean(Pos[N:Ntot,2])-Zsurf
        if count > Neqbm:
            kindx = int(Zavg/zdelta)
            zcount[min(kindx,Nbin-1)] += 1
        if count % 10000 == 0:
            PEvec[PEcount] = PEtot
            Tvec[PEcount] = count 
            PEcount += 1
        if count < Neqbm and count % 2000 == 0:
            Pdb.write(Pos)
        if count % 100000 == 0:
            atomvis.Update(Pos)
        if count > Neqbm:
            Pdb.close()
        count += 1
#        NaList, ClList, NaClList = projlibmc.calcdistances(Pos, N1, N2, L)
#        Nacount, Clcount, NaClcount = CalcRDF(NaList, ClList, NaClList, Nrbin, rbinvec, Nacount, Clcount, NaClcount)
#        #print len(NaList), len(ClList), len(NaClList)
#        Nobs += 1
        
#gNa = np.zeros(Nrbin-1, dtype = float)
#gCl = np.zeros(Nrbin-1, dtype = float)
#gNaCl = np.zeros(Nrbin-1, dtype = float)
#for i in range(Nrbin-1):
#    gNa[i] = Nacount[i] * Volume * 3. / (N1 * Nobs * 4. * scipy.pi * (rbinvec[i+1]**3. - rbinvec[i]**3.))
#    gCl[i] = Clcount[i] * Volume * 3. / (N1 * Nobs * 4. * scipy.pi * (rbinvec[i+1]**3. - rbinvec[i]**3.))
#    gNaCl[i] = NaClcount[i] * Volume * 3. / (N2 * Nobs * 4. * scipy.pi * (rbinvec[i+1]**3. - rbinvec[i]**3.))    
    print Naccept*100.0 / float(Ntotstep)
    plt.plot(Tvec, PEvec)
    countmat[:,indx] = zcount       #countmat holds the counts for all zj as a function of z